/*
 *  GuiLoader, a GuiXml loader library
 *  Copyright (c) 2006 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef GUILOADER_CONTAINERS_H
#define GUILOADER_CONTAINERS_H

#include "guiloader_impl.h"

typedef struct _Place Place;

struct _Place {
	int a, b, c, d;
};

// CrowChild -----------------------------------------------------------------

#define CROW_TYPE_CHILD (crow_child_get_type())
#define CROW_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_CHILD, CrowChild))
#define CROW_IS_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_CHILD))

typedef struct _CrowChild CrowChild;

struct _CrowChild {
	GObject parent;
	GtkWidget * widget;
	Place place;
};

GType crow_child_get_type(void);
void crow_child_init(CrowChild * child);

// CrowRowChild --------------------------------------------------------------

#define CROW_TYPE_ROW_CHILD (crow_row_child_get_type())
#define CROW_ROW_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_ROW_CHILD, CrowRowChild))
#define CROW_IS_ROW_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_ROW_CHILD))

typedef struct _CrowChild CrowRowChild;

GType crow_row_child_get_type(void);

// CrowXYChild ---------------------------------------------------------------

#define CROW_TYPE_XY_CHILD (crow_xy_child_get_type())
#define CROW_XY_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_XY_CHILD, CrowXYChild))
#define CROW_IS_XY_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_XY_CHILD))

typedef struct _CrowChild CrowXYChild;

GType crow_xy_child_get_type(void);

// CrowBoxChild --------------------------------------------------------------

#define CROW_TYPE_BOX_CHILD (crow_box_child_get_type())
#define CROW_BOX_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_BOX_CHILD, CrowBoxChild))
#define CROW_IS_BOX_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_BOX_CHILD))

typedef struct _CrowBoxChild CrowBoxChild;

struct _CrowBoxChild {
	CrowChild parent;
	CrowPackOptions options;
	unsigned padding;
};

GType crow_box_child_get_type(void);
void crow_box_child_init(CrowBoxChild * child);

// CrowButtonBoxChild --------------------------------------------------------

#define CROW_TYPE_BUTTON_BOX_CHILD (crow_button_box_child_get_type())
#define CROW_BUTTON_BOX_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_BUTTON_BOX_CHILD, CrowButtonBoxChild))
#define CROW_IS_BUTTON_BOX_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_BUTTON_BOX_CHILD))

typedef struct _CrowButtonBoxChild CrowButtonBoxChild;

struct _CrowButtonBoxChild {
	CrowBoxChild parent;
	gboolean secondary;
};

GType crow_button_box_child_get_type(void);
void crow_button_box_child_init(CrowButtonBoxChild * child);

// CrowPanedChild ------------------------------------------------------------

#define CROW_TYPE_PANED_CHILD (crow_paned_child_get_type())
#define CROW_PANED_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_PANED_CHILD, CrowPanedChild))
#define CROW_IS_PANED_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_PANED_CHILD))

typedef struct _CrowPanedChild CrowPanedChild;

struct _CrowPanedChild {
	CrowChild parent;
	gboolean resize;
	gboolean shrink;
};

GType crow_paned_child_get_type(void);
void crow_paned_child_init(CrowPanedChild * child);

// CrowNotebookChild ---------------------------------------------------------

#define CROW_TYPE_NOTEBOOK_CHILD (crow_notebook_child_get_type())
#define CROW_NOTEBOOK_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_NOTEBOOK_CHILD, CrowNotebookChild))
#define CROW_IS_NOTEBOOK_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_NOTEBOOK_CHILD))

typedef struct _CrowNotebookChild CrowNotebookChild;

struct _CrowNotebookChild {
	CrowChild parent;
	const gchar * tabLabel;
	GtkWidget * tabLabelWidget;
	const gchar * menuLabel;
	GtkWidget * menuLabelWidget;
	CrowPackOptions packOptions;
	gboolean detachable;
	gboolean reorderable;
};

GType crow_notebook_child_get_type(void);
void crow_notebook_child_init(CrowNotebookChild * child);

// CrowTableChild ------------------------------------------------------------

#define CROW_TYPE_TABLE_CHILD (crow_table_child_get_type())
#define CROW_TABLE_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_TABLE_CHILD, CrowTableChild))
#define CROW_IS_TABLE_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_TABLE_CHILD))

typedef struct _CrowTableChild CrowTableChild;

struct _CrowTableChild {
	CrowChild parent;
	unsigned paddingX;
	unsigned paddingY;
	gboolean xExpand;
	gboolean xFill;
	gboolean xShrink;
	gboolean yExpand;
	gboolean yFill;
	gboolean yShrink;
};

GType crow_table_child_get_type(void);
void crow_table_child_init(CrowTableChild * child);

// -----------------------------------------------------------------------------

typedef CrowChild * PCrowChild;

gboolean ContainerIsEmpty(GtkContainer * container);

gboolean GetExpand(CrowPackOptions packOptions);
gboolean GetFill(CrowPackOptions packOptions);
GtkAttachOptions GetAttachOptions(gboolean expand, gboolean fill, gboolean shrink);

GtkWidget * CreatePlaceholder(void);
gboolean ExposePlaceholder(GtkWidget * widget, GdkEventExpose * event, gpointer data);

void ContainerFixedSet(GtkFixed * fixed, PtrArray * children);
void ContainerLayoutSet(GtkLayout * layout, PtrArray * children);
void ContainerBinSet(GtkBin * bin, PtrArray * children);
void ContainerScrolledWindowSet(GtkScrolledWindow * scroll, PtrArray * children);
void ContainerDialogSet(GtkDialog * dialog, PtrArray * children);
void ContainerPanedSet(GtkPaned * paned, PtrArray * children);
void ContainerBoxSet(GtkBox * box, PtrArray * children, int capS, int capE);
void ContainerButtonBoxSet(GtkButtonBox * box, PtrArray * children, int capS, int capE);
void ContainerNotebookSet(GtkNotebook * notebook, PtrArray * children, int cap);
void ContainerTableSet(GtkTable * table, PtrArray * children, int capX, int capY);

#endif
