/*
 *  GuiLoader, a GuiXml loader library
 *  Copyright (c) 2006 Maxim Udushlivy
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef GUILOADER_PROXIES_H
#define GUILOADER_PROXIES_H

#include "guiloader_impl.h"

void RegisterProxies(GuiLoaderInit * init);
void RegisterGType(GType type);

void GObjectProxy(Proxy * proxy);
void PropertyGObjectSignals(PropertyFuncData * data);

void GtkWidgetProxy(Proxy * proxy);
void GtkWidgetInit(GObject * object, Node * node);
void PropertyGtkWidgetSizeRequest(PropertyFuncData * data);
void PropertyGtkWidgetEvents(PropertyFuncData * data);
void GtkWidgetAddReceivesDefaultProperty(Proxy * proxy);
void GtkWidgetAddCanFocusProperty(Proxy * proxy);
void GtkWidgetAddCanDefaultProperty(Proxy * proxy);
void GtkWidgetAddAppPaintableProperty(Proxy * proxy);
void GtkWidgetAddEventProperties(Proxy * proxy);

void GtkContainerProxyBase(Proxy * proxy);

void GtkContainerProxy(Proxy * proxy);
void GtkContainerSetChildrenPropertyFunc(Proxy * proxy, PropertyFunc func);
gboolean GtkContainerVerifyChild(GtkWidget * container, GtkWidget * child);
void PropertyGtkContainerFocusChain(PropertyFuncData * data);
void PropertyGtkContainerFocusChild(PropertyFuncData * data);

void CrowChildProxy(Proxy * proxy);
void PropertyCrowChildWidget(PropertyFuncData * data);
void CrowChildAddIndexProperty(Proxy * proxy);
void PropertyCrowChildIndex(PropertyFuncData * data);

void GtkBinProxy(Proxy * proxy);
void PropertyGtkBinChildren(PropertyFuncData * data);

void GtkAlignmentProxy(Proxy * proxy);

void GtkFrameProxy(Proxy * proxy);
void PropertyGtkFrameLabelWidgetSet(PropertyFuncData * data);
void PropertyGtkFrameLabel(PropertyFuncData * data);

void GtkAspectFrameProxy(Proxy * proxy);
void GtkAspectFrameInit(GObject * object, Node * node);

void GtkExpanderProxy(Proxy * proxy);
void GtkExpanderInit(GObject * object, Node * node);
void PropertyGtkExpanderLabel(PropertyFuncData * data);

void GtkScrolledWindowProxy(Proxy * proxy);
void PropertyGtkScrolledWindowChildren(PropertyFuncData * data);
void PropertyGtkScrolledWindowAutoViewport(PropertyFuncData * data);

void GtkHandleBoxProxy(Proxy * proxy);
void GtkHandleBoxInit(GObject * object, Node * node);

void GtkViewportProxy(Proxy * proxy);

void CrowBoxChildProxy(Proxy * proxy);
void PropertyCrowBoxChildPack(PropertyFuncData * data);
void PropertyCrowBoxChildOptions(PropertyFuncData * data);
void PropertyCrowBoxChildPadding(PropertyFuncData * data);

void GtkBoxProxy(Proxy * proxy);
void PropertyGtkBoxChildren(PropertyFuncData * data);
void PropertyGtkBoxCapacity(PropertyFuncData * data);

void CrowButtonBoxChildProxy(Proxy * proxy);
void PropertyCrowButtonBoxChildSecondary(PropertyFuncData * data);

void GtkButtonBoxProxy(Proxy * proxy);
void PropertyGtkButtonBoxChildren(PropertyFuncData * data);
void PropertyGtkButtonBoxCapacity(PropertyFuncData * data);

void CrowPanedChildProxy(Proxy * proxy);
void PropertyCrowPanedChildResize(PropertyFuncData * data);
void PropertyCrowPanedChildShrink(PropertyFuncData * data);

void GtkPanedProxy(Proxy * proxy);
void PropertyGtkPanedChildren(PropertyFuncData * data);

void CrowNotebookChildProxy(Proxy * proxy);
void PropertyCrowNotebookChildTabLabel(PropertyFuncData * data);
void PropertyCrowNotebookChildTabLabelWidget(PropertyFuncData * data);
void PropertyCrowNotebookChildMenuLabel(PropertyFuncData * data);
void PropertyCrowNotebookChildMenuLabelWidget(PropertyFuncData * data);
void PropertyCrowNotebookChildPackOptions(PropertyFuncData * data);
void PropertyCrowNotebookChildDetachable(PropertyFuncData * data);
void PropertyCrowNotebookChildReorderable(PropertyFuncData * data);

void GtkNotebookProxy(Proxy * proxy);
void PropertyGtkNotebookChildren(PropertyFuncData * data);
void PropertyGtkNotebookCapacity(PropertyFuncData * data);

void CrowTableChildProxy(Proxy * proxy);
void PropertyCrowTableChildCell(PropertyFuncData * data);
void PropertyCrowTableChildCellSpan(PropertyFuncData * data);
void PropertyCrowTableChildPaddingX(PropertyFuncData * data);
void PropertyCrowTableChildPaddingY(PropertyFuncData * data);
void PropertyCrowTableChildXExpand(PropertyFuncData * data);
void PropertyCrowTableChildXFill(PropertyFuncData * data);
void PropertyCrowTableChildXShrink(PropertyFuncData * data);
void PropertyCrowTableChildYExpand(PropertyFuncData * data);
void PropertyCrowTableChildYFill(PropertyFuncData * data);
void PropertyCrowTableChildYShrink(PropertyFuncData * data);

void GtkTableProxy(Proxy * proxy);
GObject * GtkTableCreate(GuiLoader * loader, Node * node);
void PropertyGtkTableChildren(PropertyFuncData * data);
void PropertyGtkTableCapacity(PropertyFuncData * data);

void CrowXYChildProxy(Proxy * proxy);
void PropertyCrowXYChildXY(PropertyFuncData * data);

void GtkAdjustmentProxy(Proxy * proxy);

void GtkWindowProxy(Proxy * proxy);
GObject * GtkWindowCreate(GuiLoader * loader, Node * node);
gboolean GtkWindowOnDeleteEvent(GObject * window);
void PropertyGtkWindowDefaultSize(PropertyFuncData * data);
void PropertyGtkWindowAccelGroups(PropertyFuncData * data);
void PropertyGtkWindowDefault(PropertyFuncData * data);

// CrowDialogButtonEntry -----------------------------------------------------

#define CROW_TYPE_DIALOG_BUTTON_ENTRY (crow_dialog_button_entry_get_type())
#define CROW_DIALOG_BUTTON_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_DIALOG_BUTTON_ENTRY, CrowDialogButtonEntry))
#define CROW_IS_DIALOG_BUTTON_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_DIALOG_BUTTON_ENTRY))

typedef struct _CrowDialogButtonEntry CrowDialogButtonEntry;

struct _CrowDialogButtonEntry {
	GObject parent;
	GtkWidget * widget;
	GtkResponseType response;
	gboolean secondary;
};

GType crow_dialog_button_entry_get_type(void);
void crow_dialog_button_entry_init(CrowDialogButtonEntry * entry);

// -----------------------------------------------------------------------------

void CrowDialogButtonEntryProxy(Proxy * proxy);
void PropertyCrowDialogButtonEntryWidget(PropertyFuncData * data);
void PropertyCrowDialogButtonEntryResponse(PropertyFuncData * data);
void PropertyCrowDialogButtonEntrySecondary(PropertyFuncData * data);

void GtkDialogProxy(Proxy * proxy);
void PropertyGtkDialogChildren(PropertyFuncData * data);
void PropertyGtkDialogButtons(PropertyFuncData * data);
void PropertyGtkDialogDefaultResponse(PropertyFuncData * data);

void GtkColorSelectionDialogProxy(Proxy * proxy);
void GtkColorSelectionDialogInit(GObject * object, Node * node);
void PropertyGtkColorSelectionDialogColorsel(PropertyFuncData * data);

void GtkFileFilterProxy(Proxy * proxy);
void PropertyGtkFileFilterName(PropertyFuncData * data);
void PropertyGtkFileFilterAddPixbufFormats(PropertyFuncData * data);
void PropertyGtkFileFilterPatterns(PropertyFuncData * data);
void PropertyGtkFileFilterMimeTypes(PropertyFuncData * data);

void GtkFileChooserProxyBase(Proxy * proxy);
void PropertyGtkFileChooserBaseFilter(PropertyFuncData * data);
void PropertyGtkFileChooserBaseFilters(PropertyFuncData * data);

void GtkFileChooserDialogProxy(Proxy * proxy);
GObject * GtkFileChooserDialogCreate(GuiLoader * loader, Node * node);

void GtkFontSelectionDialogProxy(Proxy * proxy);
void PropertyGtkFontSelectionDialogFontSelection(PropertyFuncData * data);

void GtkStatusbarProxy(Proxy * proxy);

// CrowTooltipsEntry ---------------------------------------------------------

#define CROW_TYPE_TOOLTIPS_ENTRY (crow_tooltips_entry_get_type())
#define CROW_TOOLTIPS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_TOOLTIPS_ENTRY, CrowTooltipsEntry))
#define CROW_IS_TOOLTIPS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_TOOLTIPS_ENTRY))

typedef struct _CrowTooltipsEntry CrowTooltipsEntry;

struct _CrowTooltipsEntry {
	GObject parent;
	GtkWidget * widget;
	const gchar * tip_text;
	const gchar * tip_private;
};

GType crow_tooltips_entry_get_type(void);
void crow_tooltips_entry_init(CrowTooltipsEntry * entry);

// -----------------------------------------------------------------------------

void CrowTooltipsEntryProxy(Proxy * proxy);
void PropertyCrowTooltipsEntryWidget(PropertyFuncData * data);
void PropertyCrowTooltipsEntryTipText(PropertyFuncData * data);
void PropertyCrowTooltipsEntryTipPrivate(PropertyFuncData * data);

void GtkTooltipsProxy(Proxy * proxy);
void PropertyGtkTooltipsEnabled(PropertyFuncData * data);
void PropertyGtkTooltipsTips(PropertyFuncData * data);

void GtkActionGroupProxy(Proxy * proxy);
GObject * GtkActionGroupCreate(GuiLoader * loader, Node * node);
void PropertyGtkActionGroupActions(PropertyFuncData * data);

void GtkActionProxy(Proxy * proxy);
GObject * GtkActionCreate(GuiLoader * loader, Node * node);
void PropertyGtkActionAccelerator(PropertyFuncData * data);

void GtkToggleActionProxy(Proxy * proxy);
GObject * GtkToggleActionCreate(GuiLoader * loader, Node * node);

void GtkRadioActionProxy(Proxy * proxy);
GObject * GtkRadioActionCreate(GuiLoader * loader, Node * node);
void PropertyGtkRadioActionGroup(PropertyFuncData * data);

void GtkUIManagerProxy(Proxy * proxy);
void PropertyGtkUIManagerActionGroups(PropertyFuncData * data);
void PropertyGtkUIManagerUiDefinitions(PropertyFuncData * data);

// CrowUIDefinition ----------------------------------------------------------

#define CROW_TYPE_UI_DEFINITION (crow_ui_definition_get_type())
#define CROW_UI_DEFINITION(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_UI_DEFINITION, CrowUIDefinition))
#define CROW_IS_UI_DEFINITION(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_UI_DEFINITION))

typedef struct _CrowUIDefinition CrowUIDefinition;

struct _CrowUIDefinition {
	GObject parent;
	const gchar * ui;
};

GType crow_ui_definition_get_type(void);
void crow_ui_definition_init(CrowUIDefinition * definition);

// -----------------------------------------------------------------------------

void CrowUIDefinitionProxy(Proxy * proxy);
void PropertyCrowUIDefinitionUi(PropertyFuncData * data);

void CrowUIElementProxy(Proxy * proxy);
GObject * CrowUIElementCreate(GuiLoader * loader, Node * node);

void GtkMenuBarProxy(Proxy * proxy);

void GtkToolbarProxy(Proxy * proxy);
void PropertyGtkToolbarToolbarStyleSet(PropertyFuncData * data);
void PropertyGtkToolbarTooltips(PropertyFuncData * data);
void PropertyGtkToolbarIconSizeSet(PropertyFuncData * data);

// CrowRadioGroup ------------------------------------------------------------

#define CROW_TYPE_RADIO_GROUP (crow_radio_group_get_type())
#define CROW_RADIO_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_RADIO_GROUP, CrowRadioGroup))
#define CROW_IS_RADIO_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_RADIO_GROUP))

typedef struct _CrowRadioGroup CrowRadioGroup;

struct _CrowRadioGroup {
	GObject parent;
	GSList * list;
};

GType crow_radio_group_get_type(void);
void crow_radio_group_init(CrowRadioGroup * group);

// -----------------------------------------------------------------------------

void CrowRadioGroupProxy(Proxy * proxy);
void GtkMiscProxy(Proxy * proxy);

void GtkLabelProxy(Proxy * proxy);

void GtkButtonProxyBase(Proxy * proxy);

void GtkButtonProxy(Proxy * proxy);
void PropertyGtkButtonStockId(PropertyFuncData * data);

void GtkToggleButtonProxy(Proxy * proxy);

void GtkCheckButtonProxy(Proxy * proxy);

void GtkRadioButtonProxy(Proxy * proxy);
void PropertyGtkRadioButtonGroup(PropertyFuncData * data);

void GtkEntryCompletionProxy(Proxy * proxy);
void PropertyGtkEntryCompletionStrings(PropertyFuncData * data);

void GtkEntryProxy(Proxy * proxy);
void PropertyGtkEntryCompletion(PropertyFuncData * data);
void PropertyGtkEntryInnerBorderSet(PropertyFuncData * data);

void GtkComboBoxProxy(Proxy * proxy);
GObject * GtkComboBoxCreate(GuiLoader * loader, Node * node);
void PropertyGtkComboBoxStrings(PropertyFuncData * data);

void GtkComboBoxEntryProxy(Proxy * proxy);
GObject * GtkComboBoxEntryCreate(GuiLoader * loader, Node * node);
void PropertyGtkComboBoxEntryText(PropertyFuncData * data);
void PropertyGtkComboBoxEntryEntry(PropertyFuncData * data);

void GtkSpinButtonProxy(Proxy * proxy);

void GtkRangeProxy(Proxy * proxy);

void GtkScaleProxy(Proxy * proxy);

void GtkProgressBarProxy(Proxy * proxy);

void GtkImageProxy(Proxy * proxy);

void GtkSeparatorProxy(Proxy * proxy);

void GtkSizeGroupProxy(Proxy * proxy);
void PropertyGtkSizeGroupWidgets(PropertyFuncData * data);

void GtkTextViewProxy(Proxy * proxy);
void PropertyGtkTextViewText(PropertyFuncData * data);

void GtkIconViewProxy(Proxy * proxy);

void GtkCellViewProxy(Proxy * proxy);

void GtkTreeViewProxy(Proxy * proxy);
void PropertyGtkTreeViewSelectionMode(PropertyFuncData * data);
void PropertyGtkTreeViewSearchEntry(PropertyFuncData * data);

void GtkColorButtonProxy(Proxy * proxy);

void GtkColorSelectionProxy(Proxy * proxy);

void GtkFileChooserButtonProxy(Proxy * proxy);
GObject * GtkFileChooserButtonCreate(GuiLoader * loader, Node * node);

void GtkFileChooserWidgetProxy(Proxy * proxy);
GObject * GtkFileChooserWidgetCreate(GuiLoader * loader, Node * node);

void GtkFontButtonProxy(Proxy * proxy);

void GtkFontSelectionProxy(Proxy * proxy);

void GtkFixedProxy(Proxy * proxy);
void PropertyGtkFixedChildren(PropertyFuncData * data);

void GtkLayoutProxy(Proxy * proxy);
void PropertyGtkLayoutChildren(PropertyFuncData * data);
void PropertyGtkLayoutSize(PropertyFuncData * data);

void GtkScrollbarProxy(Proxy * proxy);

void GtkEventBoxProxy(Proxy * proxy);

void GtkDrawingAreaProxy(Proxy * proxy);

void GtkCalendarProxy(Proxy * proxy);

void GtkArrowProxy(Proxy * proxy);

// CrowCustom ----------------------------------------------------------------

#define CROW_TYPE_CUSTOM (crow_custom_get_type())
#define CROW_CUSTOM(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_CUSTOM, CrowCustom))
#define CROW_CUSTOM_CLASS(meta) (G_TYPE_CHECK_CLASS_CAST((meta), CROW_TYPE_CUSTOM, CrowCustomClass))
#define CROW_IS_CUSTOM(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_CUSTOM))
#define CROW_IS_CUSTOM_CLASS(meta) (G_TYPE_CHECK_CLASS_TYPE((meta), CROW_TYPE_CUSTOM))
#define CROW_CUSTOM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), CROW_TYPE_CUSTOM, CrowCustomClass))

typedef struct _CrowCustom CrowCustom;
typedef struct _CrowCustomClass CrowCustomClass;

struct _CrowCustom {
	GtkDrawingArea parent;
};

struct _CrowCustomClass {
	GtkDrawingAreaClass parent;
};

GType crow_custom_get_type(void);
void crow_custom_class_init(CrowCustomClass * meta);
GtkWidget * crow_custom_new(void);
void crow_custom_size_request(GtkWidget * widget, GtkRequisition * requisition);
gint crow_custom_expose(GtkWidget * widget, GdkEventExpose * event);

// -----------------------------------------------------------------------------

void CrowCustomProxy(Proxy * proxy);
GObject * CrowCustomCreate(GuiLoader * loader, Node * node);

void GtkStatusIconProxy(Proxy * proxy);
GObject * GtkStatusIconCreate(GuiLoader * loader, Node * node);
void PropertyGtkStatusIconTooltip(PropertyFuncData * data);

void GtkLinkButtonProxy(Proxy * proxy);

void GtkRecentFilterProxy(Proxy * proxy);
void PropertyGtkRecentFilterName(PropertyFuncData * data);
void PropertyGtkRecentFilterAddPixbufFormats(PropertyFuncData * data);
void PropertyGtkRecentFilterPatterns(PropertyFuncData * data);
void PropertyGtkRecentFilterMimeTypes(PropertyFuncData * data);
void PropertyGtkRecentFilterApplications(PropertyFuncData * data);
void PropertyGtkRecentFilterGroups(PropertyFuncData * data);
void PropertyGtkRecentFilterAge(PropertyFuncData * data);

void GtkRecentChooserProxyBase(Proxy * proxy);
void GtkRecentChooserBaseInit(GObject * object, Node * node);
void PropertyGtkRecentChooserBaseFilter(PropertyFuncData * data);
void PropertyGtkRecentChooserBaseFilters(PropertyFuncData * data);

void GtkRecentChooserWidgetProxy(Proxy * proxy);
void GtkRecentChooserDialogProxy(Proxy * proxy);

#endif
